/**
    WIN32Theme - a GNUstep theme that integrates with uxtheme 
    Copyright (C) 2007 Christopher Armstrong 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "WIN32Color.h"
#include <windows.h>

static inline NSColor *Win32ToGSColor(int nIndex)
{
  DWORD color;
  color = GetSysColor(nIndex);
  float red, green, blue;

  red = ((float)GetRValue(color)) / 255.0;
  green = ((float)GetGValue(color)) / 255.0;
  blue = ((float)GetBValue(color)) / 255.0;
  //NSLog(@"Requested color:%d %u->(%f:%f:%f)", nIndex, color, red, green, blue);
  return [NSColor colorWithDeviceRed:red
      green:green
      blue:blue
      alpha:1.0];
  
}

@implementation NSColor (Win32ColorTheme)

+ (NSColor*) windowBackgroundColor
{
  return Win32ToGSColor(COLOR_BTNFACE);
}

+ (NSColor*) controlColor
{
  return Win32ToGSColor(COLOR_BTNFACE);
}

+ (NSColor*) controlBackgroundColor
{
  //return Win32ToGSColor(COLOR_BTNFACE);
  return [self whiteColor];
}

+ (NSColor*) controlShadowColor
{
  return Win32ToGSColor(COLOR_3DSHADOW);
}

+ (NSColor*) controlDarkShadowColor
{
  return Win32ToGSColor(COLOR_3DDKSHADOW);
}

+ (NSColor*) controlHighlightColor
{
  return Win32ToGSColor(COLOR_3DHILIGHT);
}

+ (NSColor*) controlLightHighlightColor
{
  return Win32ToGSColor(COLOR_3DLIGHT);
}

+ (NSColor*) menuBackgroundColor
{
  return Win32ToGSColor(COLOR_MENU);
}

+ (NSColor*) selectedMenuItemColor
{
  return Win32ToGSColor(COLOR_HIGHLIGHT);
}

+ (NSColor*) selectedTextBackgroundColor
{
  return Win32ToGSColor(COLOR_HIGHLIGHT);
}

+ (NSColor*) selectedMenuItemTextColor
{
  return Win32ToGSColor(COLOR_3DHILIGHT);
}

+ (NSColor*) windowFrameColor
{
  return Win32ToGSColor(COLOR_WINDOWFRAME);
}

+ (NSColor*) windowFrameTextColor
{
  return Win32ToGSColor(COLOR_CAPTIONTEXT);
}

+ (NSColor*) scrollBarColor
{
  return Win32ToGSColor(COLOR_SCROLLBAR);
}

+ (NSColor*) knobColor
{
  return Win32ToGSColor(COLOR_3DFACE);
}

+ (NSColor*) controlTextColor
{
  return Win32ToGSColor(COLOR_WINDOWTEXT);  
}

+ (NSColor*) disabledControlTextColor
{
  return Win32ToGSColor(COLOR_GRAYTEXT);
}
@end

