/**
    WIN32Theme - a GNUstep theme that integrates with uxtheme 
    Copyright (C) 2007 Christopher Armstrong 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef WIN32THEME_H
#define WIN32THEME_H

#include <GNUstepGUI/GSTheme.h>
#include "config.h"

@class GSTheme;

NSString* WIN32ThemeException;

typedef struct WIN32ThemeMargins
{
  int leftWidth, rightWidth, topHeight, bottomHeight;
} WIN32ThemeMargins;

typedef enum WIN32ThemeSizeType
{
  WIN32ThemeSizeBestFit,
  WIN32ThemeSizeMinimum,
  WIN32ThemeSizeDraw
} WIN32ThemeSizeType;

WIN32ThemeMargins WIN32ZeroThemeMargins;

@interface WIN32Theme : GSTheme
{
  BOOL platformSupportsVS;
  BOOL userEnabledVS;
}

+ (void) initialize;
- (void) activate;

/**
  * Returns true if the platform supports visual styles (Windows XP and Windows Vista).
  */
- (BOOL) platformCanUseVisualStyles;

/**
  * Tests whether we should attempt to retrieve a visual style. It will always return false if
  * -platformCanUseVisualStyles is false or if the user has disabled it with a user default setting.
  */
- (BOOL) useVisualStyles;

/**
 * On platforms that are capable of using visual styles, this function returns
 * a boolean indicating whether or not a visual style is active. It is needed
 * on platforms that support visual styles but can turn visual styles off.
 */
- (BOOL) isThemeActive;

- (BOOL) drawThemeBackground:(HTHEME)hTheme
		      inRect:(NSRect)rect
		        part:(int)part
		       state:(int)state;

- (BOOL) isTheme:(HTHEME)hTheme 
     partDefined:(int)part;

- (NSSize) defaultSizeForTheme:(HTHEME)hTheme 
                          part:(int)part 
                         state:(int)state;

- (int) intForTheme:(HTHEME)hTheme 
               part:(int)part 
              state:(int)state 
           property:(int)property;

- (int) metricForTheme:(HTHEME)hTheme 
                  part:(int)part 
                 state:(int)state 
              property:(int)property;

- (NSRect) backgroundContentRectForTheme:(HTHEME)hTheme
                                    part:(int)part
                                   state:(int)state
                                   frame:(NSRect)frame; 

- (WIN32ThemeMargins) marginsForTheme:(HTHEME)hTheme
                                 part:(int)part
                                state:(int)state;
- (NSSize) sizeForTheme:(HTHEME)hTheme
                   part:(int)part
                  state:(int)state
                  type:(WIN32ThemeSizeType)size; 

- (HTHEME) themeWithClassName:(NSString*)themeClass;
- (void) releaseTheme:(HTHEME)theme;

@end


  
#endif
