/**
    WIN32Theme - a GNUstep theme that integrates with uxtheme 
    Copyright (C) 2007 Christopher Armstrong 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <GNUstepGUI/GSTheme.h>
#include <AppKit/NSWindow.h>
#include <AppKit/NSView.h>
#include <AppKit/NSGraphics.h>
#include <AppKit/NSGraphicsContext.h>
#include <Foundation/NSAffineTransform.h>
#include <AppKit/NSAffineTransform.h>
#include <AppKit/NSImageRep.h>
#include <AppKit/NSImage.h>
#include <Foundation/NSException.h>

#include "WIN32Theme.h"
#include "WIN32VSImageRep.h"

#include <limits.h>

WIN32ThemeMargins WIN32ZeroThemeMargins = { 0, 0, 0, 0};
NSString *WIN32ThemeException = @"WIN32ThemeException";

@implementation WIN32Theme

+ (void) initialize
{
  NSLog(@"WIN32 Theme (C) Christopher Armstrong 2007");
}

- (id) initWithBundle:(NSBundle*)bundle
{
  self = [super initWithBundle:bundle];
  if (!self)
    return nil;
  NSLog(@"Initializing theme.");

  platformSupportsVS = NO;
  userEnabledVS = YES;
  return self;
}

- (void) activate
{
  [super activate];
}

- (HTHEME) themeWithClassName:(NSString*)className
{
  return NULL;
}

- (void) releaseTheme:(HTHEME)hTheme
{
  [NSException raise:WIN32ThemeException
              format:@"WIN32Theme -releaseTheme called on a platform that doesn't support visual styles. (hTheme=%p)",
              hTheme];
}

- (BOOL) isTheme:(HTHEME)hTheme partDefined:(int)part
{
  return NO;
}  

- (BOOL) useVisualStyles
{
  return (platformSupportsVS && userEnabledVS && [self isThemeActive]) ? YES : NO;
}

- (BOOL) platformCanUseVisualStyles
{
  return platformSupportsVS;
}

- (BOOL) isThemeActive
{
  return NO;
}

- (BOOL) drawThemeBackground:(HTHEME)themeClass
		      inRect:(NSRect)rect
		      part:(int)part
		      state:(int)state
{
  return NO;
}

- (NSSize) defaultSizeForTheme:(HTHEME)hTheme part:(int)part state:(int)state
{
  [NSException raise:WIN32ThemeException
              format:@"WIN32Theme -defaultSizeForTheme:part:style: called on a platform that doesn't support visual styles. (hTheme=%p)",
              hTheme];
  return NSZeroSize;
}

- (int) intForTheme:(HTHEME)hTheme part:(int)part state:(int)state 
    property:(int)property
{
  [NSException raise:WIN32ThemeException
              format:@"WIN32Theme -intForTheme:part:style: called on a platform that doesn't support visual styles. (hTheme=%p)",
              hTheme];
  return NSNotFound;
}

- (int) metricForTheme:(HTHEME)hTheme part:(int)part state:(int)state 
    property:(int)property
{
  [NSException raise:WIN32ThemeException
              format:@"WIN32Theme -metricForTheme:part:style: called on a platform that doesn't support visual styles. (hTheme=%p)",
              hTheme];
  return NSNotFound;
}

- (NSRect) backgroundContentRectForTheme:(HTHEME)hTheme
                                    part:(int)part
                                   state:(int)state
                                   frame:(NSRect)frame 
{
  [NSException raise:WIN32ThemeException
    format:@"WIN32Theme -backgroundContentRectForTheme:part:state:frame: called on a platform that doesn't support visual"
    "styles. (hTheme=%p)", hTheme];
  return  NSZeroRect;
}

- (WIN32ThemeMargins) marginsForTheme:(HTHEME)hTheme
                                 part:(int)part
                                state:(int)state;

{
  [NSException raise:WIN32ThemeException
    format:@"WIN32Theme -marginsForTheme:part:state called on a platform that doesn't support visual"
    "styles. (hTheme=%p)", hTheme];
  return WIN32ZeroThemeMargins;
}
- (NSSize) sizeForTheme:(HTHEME)hTheme
                   part:(int)part
                  state:(int)state
                  type:(WIN32ThemeSizeType)size 
{
  [NSException raise:WIN32ThemeException
    format:@"WIN32Theme -sizeForTheme:part:state:type: called on a platform that doesn't support visual"
    "styles. (hTheme=%p)", hTheme];
  return NSZeroSize;
}
@end

