/**
    WIN32Theme - a GNUstep theme that integrates with uxtheme 
    Copyright (C) 2007 Christopher Armstrong 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <GNUstepGUI/GSTheme.h>
#include <AppKit/NSColor.h>
#include <AppKit/NSGraphics.h>
#include <AppKit/NSGraphicsContext.h>
#include <AppKit/NSAffineTransform.h>

#undef _WIN32_WINNT
#define _WIN32_WINNT 0x501
#include <uxtheme.h>
#include <tmschema.h>

#include "WIN32Theme.h"

#include "WIN32Theme.h"

@interface WIN32Theme (Buttons)
- (void) drawButton: (NSRect)frame
  in:(NSButtonCell*)cell
  view:(NSView*)view
  style:(int)style
  state:(GSThemeControlState)state;

- (NSSize) buttonBorderForStyle: (int)style 
                          state: (GSThemeControlState)state;
@end

@implementation WIN32Theme (Buttons)


- (void) drawButton: (NSRect)frame
  in:(NSButtonCell*)cell
  view:(NSView*)view
  style:(int)style
  state:(GSThemeControlState)state
{
  HTHEME hButtonTheme;
  int drawState;
  //HDC hdc;
  //RECT rcRect;
  
  hButtonTheme = [self themeWithClassName:@"button"];
  if ([self isTheme:hButtonTheme partDefined:BP_PUSHBUTTON])
  {
    BOOL result;
    //hdc = GetCurrentHDC();
    //rcRect = WIN32RectFromFrame(frame);
  
    switch (state)
    {
      case GSThemeHighlightedState:
        drawState = PBS_DEFAULTED;
        break;
      case GSThemeSelectedState:
        drawState = PBS_PRESSED;
        break;
      case GSThemeNormalState:
      default:
        drawState = PBS_NORMAL;
        break;
    }
    
    result = [self drawThemeBackground:hButtonTheme
          inRect:frame  
          part: BP_PUSHBUTTON
          state: drawState];
    [self releaseTheme:hButtonTheme];
    if (result)
    {
      return;
    }
  }
  {
    NSColor *blacker = [NSColor blackColor];
    NSColor *black = [NSColor controlDarkShadowColor];
    NSColor *dark = [NSColor controlShadowColor];
    NSColor *grey = [NSColor controlHighlightColor];
    NSColor *white = [NSColor controlLightHighlightColor];
  
    if ([cell isOpaque])
    {
      NSColor *backgroundColor;
      switch (state)
      {
        case GSThemeNormalState:
        case GSThemeHighlightedState:
        case GSThemeSelectedState:
          backgroundColor = [NSColor controlColor];
          break;
        default:
          backgroundColor = nil;
      }
      if (backgroundColor)
      {
        [backgroundColor set];
        NSRectFill(frame);
      }
    }
    
    NSRectEdge sides[] = 
      {
        NSMaxXEdge, NSMinYEdge, NSMinXEdge, NSMaxYEdge,
        NSMaxXEdge, NSMinYEdge };
    NSRectEdge flipped_sides[] = 
      { 
        NSMaxXEdge, NSMaxYEdge, NSMinXEdge, NSMinYEdge,
        NSMaxXEdge, NSMaxYEdge };
    switch (state)
    {
    case GSThemeNormalState:
      {
  
      NSColor *colors[]= {black, black, grey, grey, dark, dark};  
      NSRectEdge sides[] = 
        {
          NSMaxXEdge, NSMinYEdge, NSMinXEdge, NSMaxYEdge,
          NSMaxXEdge, NSMinYEdge };
      NSRectEdge flipped_sides[] = 
        { 
          NSMaxXEdge, NSMaxYEdge, NSMinXEdge, NSMinYEdge,
          NSMaxXEdge, NSMaxYEdge };
  
      if ([[NSView focusView] isFlipped])
        NSDrawColorTiledRects(frame, frame, flipped_sides, colors, 6);
      else  
        NSDrawColorTiledRects(frame, frame, sides, colors, 6);
      break;
      }
    case GSThemeSelectedState:
      {
      NSColor *colors[] = { black, black, black, black, dark, dark};
  
      if ([[NSView focusView] isFlipped])
        NSDrawColorTiledRects(frame, frame, flipped_sides, colors, 6);
      else
        NSDrawColorTiledRects(frame, frame, sides, colors, 6);
      break;
      } 
    case GSThemeHighlightedState:
      {
      NSColor *colors[] = { grey, grey, black, black, white, white, dark, dark };
      NSRectEdge sides[] = 
        {
          NSMaxXEdge, NSMinYEdge, NSMinXEdge, NSMaxYEdge,
          NSMaxXEdge, NSMinYEdge, NSMinXEdge, NSMaxYEdge };
      NSRectEdge flipped_sides[] = 
        { 
          NSMaxXEdge, NSMaxYEdge, NSMinXEdge, NSMinYEdge,
          NSMaxXEdge, NSMaxYEdge, NSMinXEdge, NSMinYEdge };
  
      if ([[NSView focusView] isFlipped])
        NSDrawColorTiledRects(frame, frame, flipped_sides, colors, 8);
      else
        NSDrawColorTiledRects(frame, frame, sides, colors, 8);
      break;
      }
    }
  }
}

- (NSSize) buttonBorderForStyle: (int)style 
                          state: (GSThemeControlState)state
{
  //return NSMakeSize(3, 3);
  HTHEME hButtonTheme;
  //MARGINS margins;
  int drawState;

  hButtonTheme = [self themeWithClassName:@"button"];
  if ([self isTheme:hButtonTheme partDefined:BP_PUSHBUTTON])
  {
    WIN32ThemeMargins margins;
    switch (state)
    {
      case GSThemeHighlightedState:
        drawState = PBS_DEFAULTED;
        break;
      case GSThemeSelectedState:
        drawState = PBS_PRESSED;
        break;
      case GSThemeNormalState:
      default:
        drawState = PBS_NORMAL;
        break;
    }
    margins = [self marginsForTheme:hButtonTheme part:BP_PUSHBUTTON state:drawState];
    [self releaseTheme:hButtonTheme];

    return NSMakeSize(margins.leftWidth+margins.rightWidth, 
       margins.topHeight + margins.bottomHeight); 
  }
  else
    return NSMakeSize(3, 3);
}

@end
