/**
    WIN32Theme - a GNUstep theme that integrates with uxtheme 
    Copyright (C) 2007 Christopher Armstrong 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <GNUstepGUI/GSTheme.h>
#include <AppKit/NSMenu.h>
#include <AppKit/NSMenuItemCell.h>
#include <AppKit/NSWindow.h>
#include <AppKit/NSGraphics.h>
#include <AppKit/NSBezierPath.h>
//#include "NSColor.h"

#undef _WIN32_WINNT
#define _WIN32_WINNT 0x501

#include <uxtheme.h>
#include <tmschema.h>
#include "vsstyle.h"

#include "WIN32Theme.h"
#include "WIN32Color.h"

@implementation WIN32Theme (menus)

- (float) minimumMenuImageGutterWidth:(NSMenuView*)menuView
{
  HTHEME hMenuTheme;
  NSSize size;
  WIN32ThemeMargins checkBackground, check;
  float gutterSize;

  hMenuTheme = [self themeWithClassName:@"menu"];
  if (hMenuTheme)
  {

    checkBackground = [self marginsForTheme:hMenuTheme part:MENU_POPUPCHECKBACKGROUND state:0];
    check = [self marginsForTheme:hMenuTheme part:MENU_POPUPCHECK state:0];

    size = [self sizeForTheme:hMenuTheme part:MENU_POPUPCHECK state:0 type:WIN32ThemeSizeBestFit]; 

    gutterSize = checkBackground.leftWidth + checkBackground.rightWidth + check.leftWidth + check.rightWidth + size.width;
  }  
  else
  {
    gutterSize = 18;
  }
  
  return gutterSize;
}

- (GSBorderLayout) menuViewBorderLayout:(NSMenuView*)menuView
{
  HTHEME hMenuTheme;
  GSBorderLayout result;
  hMenuTheme = [self themeWithClassName:@"menu"];
  
  if (hMenuTheme)
  {
    if ([self isTheme:hMenuTheme partDefined:MENU_POPUPBACKGROUND])
      result = [super menuViewBorderLayout:menuView];
  }
  else if ([self useVisualStyles])
    result = GSMakeBorderLayout(3, 3, 3, 3);
  else
    result = GSMakeBorderLayout(1, 2, 1, 2);

  if (hMenuTheme) [self releaseTheme:hMenuTheme]; 
  return result;
}

- (void) drawMenuViewBackground:(NSRect)frame
                      view:(NSMenuView*)view
{
  HTHEME hMenuTheme;
  hMenuTheme = [self themeWithClassName:@"menu"];
  BOOL winXPMenus = [self useVisualStyles];

  if (hMenuTheme)
  {
    NSDebugLLog(@"WIN32Theme", @"Menu theme handle aquired");
    if ([self isTheme:hMenuTheme partDefined:MENU_POPUPBACKGROUND])
    {
      NSDebugLLog(@"WIN32Theme", @"Menu theme MENU_POPUPBACKGROUND part exists");
      NSSize size;
      WIN32ThemeMargins checkBackground, check, borderMargins, backgroundMargins, popupItemMargins;
      int gutterSize, borderSize;
      BOOL result;

      result = [self drawThemeBackground:hMenuTheme
          inRect:frame
          part:MENU_POPUPBACKGROUND
          state:0];
      
      result = [self drawThemeBackground:hMenuTheme
          inRect:frame
          part:MENU_POPUPBORDERS
          state:0];


      checkBackground = [self marginsForTheme:hMenuTheme part:MENU_POPUPCHECKBACKGROUND state:0];
      check = [self marginsForTheme:hMenuTheme part:MENU_POPUPCHECK state:0];

      size = [self sizeForTheme:hMenuTheme part:MENU_POPUPCHECK state:0 type:WIN32ThemeSizeBestFit]; 

      // Doesn't exist: borderMargins = [self marginsForTheme:hMenuTheme part:MENU_POPUPBORDERS state:0];
      backgroundMargins = [self marginsForTheme:hMenuTheme part:MENU_POPUPBACKGROUND state:0];
      popupItemMargins = [self marginsForTheme:hMenuTheme part:MENU_POPUPITEM state:0];
      borderSize = [self intForTheme:hMenuTheme part:MENU_POPUPBACKGROUND state:0 property:TMT_BORDERSIZE]; 
      NSLog(@"borderMargins={%d,%d,%d,%d}", borderMargins.leftWidth, borderMargins.rightWidth, borderMargins.topHeight, borderMargins.bottomHeight);
      NSLog(@"backgroundMargins={%d,%d,%d,%d}", backgroundMargins.leftWidth, backgroundMargins.rightWidth, backgroundMargins.topHeight, backgroundMargins.bottomHeight);
      NSLog(@"popupItemMargins={%d,%d,%d,%d}", popupItemMargins.leftWidth, popupItemMargins.rightWidth, popupItemMargins.topHeight, popupItemMargins.bottomHeight);
      NSLog(@"backgroundSize={%d}", borderSize);
      NSLog(@"gutterSize={%d}", gutterSize);

      gutterSize = checkBackground.leftWidth + checkBackground.rightWidth + check.leftWidth + check.rightWidth + size.width;
      
      frame = NSMakeRect(
          frame.origin.x + gutterSize,
          frame.origin.y,
          3, 
          frame.size.height);

      [self drawThemeBackground: hMenuTheme
        inRect: frame
        part: MENU_POPUPGUTTER
        state: 0];
      [self releaseTheme:hMenuTheme]; 
      return;
    }
    [self releaseTheme:hMenuTheme];
  }

  if (winXPMenus)
  {
    [[NSColor windowFrameColor] set];
    [NSBezierPath strokeRect:frame];
    [[NSColor menuBackgroundColor] set];
    NSRectFill(NSInsetRect(frame, 1.0, 1.0));
  }
  else
  {
    [[NSColor menuBackgroundColor] set];
    NSRectFill(frame);
    [self drawButton:frame in:nil view:view style:0 state:GSThemeNormalState];
  }
}

- (void) drawMenuItemBorderAndBackground:(NSRect)frame
  in:(NSMenuItemCell*)cell
  view:(NSView*)view
  state:(GSThemeControlState)state
{
  HTHEME hMenuTheme;
  hMenuTheme = [self themeWithClassName:@"menu"]; 
  if ([self isTheme:hMenuTheme partDefined:MENU_POPUPITEM])
  {
    BOOL result;
    int drawState = 0;
    NSLog(@"Drawing menu item.");

    if ([[cell menuItem] isEnabled])
    {
      switch (state)
      {
        case GSThemeSelectedState:
          drawState = MPI_HOT;
          break;
        case GSThemeHighlightedState:
          drawState = MPI_HOT;
          break;
        case GSThemeNormalState:
        default:
          drawState = MPI_NORMAL;
          break;
      }
    }
    else
    {
      switch (state)
      {
        case GSThemeSelectedState:
          drawState = MPI_DISABLEDHOT;
          NSLog(@"Selected.");
          break;
        case GSThemeHighlightedState:
          drawState = MPI_DISABLEDHOT;
          NSLog(@"Highlighted");
          break;
        case GSThemeNormalState:
        default:
          drawState = MPI_DISABLED;
          break;
      }
    }

    result = [self drawThemeBackground: hMenuTheme
          inRect:frame
          part:MENU_POPUPITEM
          state:drawState];
    [self releaseTheme:hMenuTheme];
    if (result)
    {
      return;
    }
  }
  else
  {
    // We only render using the background color if we are
    // in a highlighted state, as the background color on the
    // cell is usually wrong if we are in a normal state.
    if (state!=GSThemeNormalState)
    { 
      [[NSColor selectedMenuItemColor] set];
      NSRectFill(frame);
    }
  }
}

@end  

