/**
    WIN32Theme - a GNUstep theme that integrates with uxtheme 
    Copyright (C) 2007 Christopher Armstrong 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <GNUstepGUI/GSTheme.h>
#include <AppKit/NSButtonCell.h>
#include <AppKit/NSScroller.h>
#include <Foundation/NSException.h>

#undef _WIN32_WINNT 
#define _WIN32_WINNT 0x501

#include <windows.h>
#include <tmschema.h>

#include "vsstyle.h"
#include "vssym32.h"
#include "WIN32Theme.h"

@interface WIN32ScrollerButtonCell : NSButtonCell
{
  WIN32Theme* _theme;
  NSScrollerPart _part;
  BOOL _horizontal;
}

- (id) initWithTheme:(WIN32Theme*)theme scrollerPart:(NSScrollerPart)part horizontal:(BOOL)h;
- (void) dealloc;
- (void) highlight:(BOOL)flag withFrame:(NSRect)frame inView:(NSView*)view;
- (void) drawWithFrame:(NSRect)frame inView:(NSView*)view;
- (int) _partForPart:(NSScrollerPart)part;
@end

@implementation WIN32ScrollerButtonCell 
- (id) initWithTheme:(WIN32Theme*)theme scrollerPart:(NSScrollerPart)part horizontal:(BOOL)h
{
  self = [super init];
  if (!self) return nil;

  _theme = RETAIN(theme);
  _part = part;
  _horizontal = h;
  return self;
}

- (void) dealloc
{
  RELEASE(_theme);
  [super dealloc];
}

- (int) _partForPart:(NSScrollerPart)part
{
  switch (part)
  {
    case NSScrollerKnob:
      return SBP_THUMBBTNVERT;
    case NSScrollerKnobSlot:
      return SBP_UPPERTRACKVERT;
    case NSScrollerIncrementLine:
    case NSScrollerDecrementLine:  
      return SBP_ARROWBTN;
    default:
      return 0;
  }
}

- (void) highlight:(BOOL)flag withFrame:(NSRect)frame inView:(NSView*)view
{
  HTHEME hTheme = [_theme themeWithClassName:@"scrollbar"];
  
  if ([_theme isTheme:hTheme partDefined:[self _partForPart:_part]] )
  {
    NSLog(@"Draw scroller part.");
    int part;
    BOOL result;

    switch (_part)
    {
      case NSScrollerKnob:
  
        result = [_theme drawThemeBackground:hTheme
          inRect:frame
          part: _horizontal ? SBP_THUMBBTNHORZ : SBP_THUMBBTNVERT
          state: flag ? SCRBS_PRESSED : SCRBS_NORMAL
          ];
        result = [_theme drawThemeBackground:hTheme
          inRect:frame
          part: _horizontal ? SBP_GRIPPERHORZ : SBP_GRIPPERVERT
          state: flag ? SCRBS_PRESSED : SCRBS_NORMAL];
        break;
      case NSScrollerKnobSlot:
        result = [_theme drawThemeBackground:hTheme
          inRect: frame
          part: _horizontal ? SBP_UPPERTRACKHORZ : SBP_UPPERTRACKVERT
          state: flag ? SCRBS_PRESSED : SCRBS_NORMAL
          ];
        break;
      case NSScrollerIncrementLine:
      case NSScrollerDecrementLine:
        if (_part==NSScrollerIncrementLine)
        {
          if (_horizontal)
            part = flag ? ABS_RIGHTPRESSED : ABS_RIGHTNORMAL;
          else
            part = flag ? ABS_DOWNPRESSED : ABS_DOWNNORMAL;
        }
        else
        {
          if (_horizontal)
            part = flag ? ABS_LEFTPRESSED : ABS_LEFTNORMAL;
          else
            part = flag ? ABS_UPPRESSED : ABS_UPNORMAL;
            
        }
        result = [_theme drawThemeBackground:hTheme
          inRect: frame
          part: SBP_ARROWBTN
          state: part 
          ];
        break;
      default:
        break;
    }    
    [_theme releaseTheme:hTheme];
    if (result)
      return;
  }
  else
  {
    NSLog(@"No theme part defined.");
  }
}
- (void) drawWithFrame:(NSRect)frame inView:(NSView*)view
{
  [self highlight:NO withFrame:frame inView:view];
}
@end
  
@implementation WIN32Theme (Scrollbars)

- (NSCell*) cellForScrollerKnob:(BOOL)horiz
{
  HTHEME hTheme = [self themeWithClassName:@"scrollbar"];

  if (hTheme && 
      horiz ? [self isTheme:hTheme partDefined:SBP_THUMBBTNHORZ]
            : [self isTheme:hTheme partDefined:SBP_THUMBBTNVERT])
  {
    WIN32ScrollerButtonCell *cell;

    cell = [[WIN32ScrollerButtonCell alloc] initWithTheme:self scrollerPart:NSScrollerKnob
      horizontal:horiz];
    return AUTORELEASE(cell);
  }
  else
  {
    NSCell *knobSlotCell;
    knobSlotCell = [super cellForScrollerKnob:horiz];
    [knobSlotCell setImage:nil];
    return knobSlotCell;	
  }
}

- (NSCell*) cellForScrollerKnobSlot:(BOOL)horizontal
{
  HTHEME hTheme;

  hTheme = [self themeWithClassName:@"scrollbar"];
  if (hTheme && 
      horizontal ? [self isTheme:hTheme partDefined:SBP_UPPERTRACKHORZ]
                 : [self isTheme:hTheme partDefined:SBP_UPPERTRACKVERT])
  {
    WIN32ScrollerButtonCell *cell;
    NSLog(@"Get scroller knob slot.");
    cell = [[WIN32ScrollerButtonCell alloc] initWithTheme:self scrollerPart:NSScrollerKnobSlot
      horizontal:horizontal];
    return AUTORELEASE(cell);
  }
  else
  {
    return [super cellForScrollerKnobSlot:horizontal];
  }
}

- (NSCell*) cellForScrollerArrow:(NSScrollerArrow)arrow horizontal:(BOOL)h
{
  HTHEME hTheme;

  hTheme = [self themeWithClassName:@"scrollbar"];

  if (hTheme &&
      [self isTheme:hTheme 
        partDefined: SBP_ARROWBTN])
  {

    WIN32ScrollerButtonCell *cell;
    NSScrollerPart part;
    if (arrow==NSScrollerIncrementArrow)
      part = NSScrollerIncrementLine;
    else
      part = NSScrollerDecrementLine;
    cell = [[WIN32ScrollerButtonCell alloc] initWithTheme:self scrollerPart:part
      horizontal:h];
    return AUTORELEASE(cell);
  }
  else
    return [super cellForScrollerArrow:arrow horizontal:h];
}

- (float) defaultScrollerWidth
{
  HTHEME hTheme;

  hTheme = [self themeWithClassName:@"scrollbar"];

  int scrollerWidth;

  NS_DURING
  {
    scrollerWidth = [self intForTheme:hTheme 
      part:0
      state:0
      property:TMT_SCROLLBARWIDTH];
  }
  NS_HANDLER 
  {
    scrollerWidth = 16;
  }
  NS_ENDHANDLER
  if (hTheme)
    [self releaseTheme:hTheme];
  return scrollerWidth;
}

@end  
